/*
 * ptty_device.c:
 * main executable file used for device simulation
 * --------------------------
 * Author: f0rm2l1n
 */

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <pthread.h>
#include <sys/uio.h>
#include <termios.h>

#include "ptty_device.h"

int getmaster() {
  int fdm = open("/dev/ptmx", O_RDWR | O_NOCTTY);
  if (fdm < 0)
    perror("[basic] ptmx open"), exit(1);
  grantpt(fdm);
  unlockpt(fdm);
  return fdm;
}

int getslave(int fdm) {
  char buf[32];
  int ptyno, fds;
  if (ioctl(fdm, TIOCGPTN, &ptyno))
    perror("[basic] TIOCGPTN"), exit(1);
  sprintf(buf, "/dev/pts/%d", ptyno);
  fds = open(buf, O_RDWR | O_NOCTTY);
  if (fds < 0)
    perror("[basic] pts open"), exit(1);
  return fds;
}

int setserial(int fd, int ldisc) {
  struct termios ti;
  int saved_ldisc;
  unsigned char flowctl = 0;
  unsigned int speed = B115200;

  if (tcflush(fd, TCIOFLUSH) < 0) {
    perror("[basic] Failed to flush serial port");
    close(fd);
    exit(1);
  }

  if (ioctl(fd, TIOCGETD, &saved_ldisc) < 0) {
    perror("[basic] Failed get serial line discipline");
    close(fd);
    exit(1);
  }

  /* Switch TTY to raw mode */
  memset(&ti, 0, sizeof(ti));
  cfmakeraw(&ti);

  ti.c_cflag |= (speed | CLOCAL | CREAD);

  if (flowctl) {
    /* Set flow control */
    ti.c_cflag |= CRTSCTS;
  }

  if (tcsetattr(fd, TCSANOW, &ti) < 0) {
    perror("[basic] Failed to set serial port settings");
    close(fd);
    exit(1);
  }

  if (ioctl(fd, TIOCSETD, &ldisc) < 0) {
    perror("[basic] Failed set serial line discipline");
    close(fd);
    exit(1);
  }

  return 0;
}

int ax25_register(int fd) {
  setserial(fd, N_AX25);
  return 0;
}
